from rolldecayestimators import equations
from rolldecayestimators.substitute_dynamic_symbols import lambdify
import sympy as sp
from rolldecayestimators.symbols import *

#calculate_acceleration_linear = lambdify(equations.acceleration_equation_linear.rhs)
#calculate_velocity_linear = lambdify(equations.velocity_equation_linear.rhs)

